<?php

namespace Drupal\eca_twilio_action\Plugin\Action;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\eca\Plugin\Action\ConfigurableActionBase;
use Drupal\eca\Token\TokenInterface;
use Drupal\token\TokenEntityMapperInterface;
use Drupal\twilio\Services\Sms;
use Exception;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a custom action to send SMS via Twilio.
 *
 * @Action(
 *   id = "send_twilio_sms",
 *   label = @Translation("Send Twilio SMS")
 * )
 */
class SendTwilioSms extends ConfigurableActionBase {

  /**
   * The Twilio SMS service.
   *
   * @var \Drupal\twilio\Services\Sms
   */
  protected Sms $twilioSms;

  /**
   * Constructs a SendTwilioSms object.
   *
   * @param array $configuration
   *   The plugin configuration.
   * @param string $plugin_id
   *   The plugin ID.
   * @param mixed $plugin_definition
   *   The plugin definition.
   * @param \Drupal\twilio\Services\Sms $twilioSms
   *   The Twilio SMS service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   * @param \Drupal\eca\EcaState $state
   *   The state service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory service.
   * @param \Drupal\eca\Token\TokenInterface $token_services
   *   The token service.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match service.
   * @param \Drupal\token\TokenEntityMapperInterface $tokenEntityMapper
   *   The token entity mapper service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, Sms $twilioSms, EntityTypeManagerInterface $entity_type_manager, AccountProxyInterface $current_user, TimeInterface $time, \Drupal\eca\EcaState $state, LoggerChannelFactoryInterface $logger_factory, TokenInterface $token_services, RouteMatchInterface $route_match, TokenEntityMapperInterface $tokenEntityMapper) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $token_services, $current_user, $time, $state);
    $this->twilioSms = $twilioSms;
    $this->loggerFactory = $logger_factory;
    $this->routeMatch = $route_match;
    $this->tokenEntityMapper = $tokenEntityMapper;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('twilio.sms'),
      $container->get('entity_type.manager'),
      $container->get('current_user'),
      $container->get('datetime.time'),
      $container->get('eca.state'), // Correct service
      $container->get('logger.factory'),
      $container->get('eca.token_services'), // Correct token service
      $container->get('current_route_match'),
      $container->get('token.entity_mapper')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function execute(): void {
    // Replace tokens in the configuration values
    $phone_number = $this->tokenServices->replaceClear($this->configuration['phone_number']);
    $message = $this->tokenServices->replaceClear($this->configuration['message']);
    // Decode HTML entities in the message
    $decoded_message = html_entity_decode($message);

    try {
      $this->twilioSms->messageSend($phone_number, $decoded_message);

      \Drupal::logger('eca_twilio_action')->info('Message sent successfully to @phone_number', ['@phone_number' => $phone_number]);
    } catch (Exception $e) {
      \Drupal::logger('eca_twilio_action')->error('Error sending message: @message', ['@message' => $e->getMessage()]);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'phone_number' => '',
      'message' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['phone_number'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Phone Number'),
      '#default_value' => $this->configuration['phone_number'],
      '#description' => $this->t('The phone number to send the SMS to.'),
    ];
    $form['message'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Message'),
      '#default_value' => $this->configuration['message'],
      '#description' => $this->t('The message to send.'),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $this->configuration['phone_number'] = $form_state->getValue('phone_number');
    $this->configuration['message'] = $form_state->getValue('message');
    parent::submitConfigurationForm($form, $form_state);
  }

}
